<?
/*
GWebCache - Gnutella Web-Based Host Cache System
Copyright (c) 2002 by Hauke Dmpfling <hauke@zero-g.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

// visit http://www.zero-g.net/gwebcache/ or http://www.gnucleus.net/
$VERSION="GWebCache 0.6.1";

# User Vars
$enable_stats=1; # set to 1 or 0 to enable or disable statistics
$enable_url=1; # enable submitting of alternate caches?
$enable_http=0; # send outgoing HTTP requests? (very useful for URL verification!)

# The following values should be fine as they are
$maxhosts=20; # values between 5-30 
$maxurls=20; # values between 5-30

$url_self="gcache.php"; # Filename of this script

$file_data="gdata.txt"; # Data file


# ##### Script Begins #####

# Init
$warnings="";
if( $enable_url==1 && $enable_http==1 && ini_get('allow_url_fopen')!=1 )
	$enable_http=0;

# Read File
$lock = fopen($file_data.".lock","a+");
if( !flock($lock, LOCK_EX) )
	exitwarn("ERROR: File locking error");
$data = file($file_data);
for($i=0;$i<count($data);$i++) {
	$data[$i]=trim($data[$i]);
}
if(    $data[0]!=4
	|| $data[1]!=8
	|| count($data)<$data[3]
	|| $data[2]<$data[1]
	|| $data[3]<$data[2] ) { # is the file possibly corrupt?
	$data = array(4, 8, 8, 8, 0, 0, time()+3600, 0, ''); # reset!
}
$dont_count=0; # for stats

# ##### Simple Requests #####

# Ping-Pong check
# Note: could be done before reading file, but we want to make sure the file is readable first!
if( $HTTP_GET_VARS["ping"] ) {
	echo "PONG $VERSION";
	exit;
}

# Want host list
if( $HTTP_GET_VARS["hostfile"] ) {
	for($i=$data[1];$i<$data[2];$i++) {
		echo "$data[$i]\n";
	}
}

# Want URL List
else if( $HTTP_GET_VARS["urlfile"] ) {
	for($i=$data[2];$i<$data[3];$i++) {
		echo "$data[$i]\n";
	}
}

# Want Statfile
else if( $HTTP_GET_VARS["statfile"] ) {
	$dont_count=1;
	echo $data[4]."\n";
	echo $data[7]."\n";
}

# Want user-readable stats
else if( $HTTP_GET_VARS["stats"] ) {
	$dont_count=1;
	print_stats();
}

# ##### Update Request #####
else {

# Retrive ip1,ip2,... and url1,url2,.. variables
unset($gip, $gurl);
while (list($key, $value) = each ($HTTP_GET_VARS)) {
	if( strncmp("ip",$key,2)==0 && strlen($value)>0 )
		$gip[]=$value;
	if( strncmp("url",$key,3)==0 && strlen($value)>0 )
		$gurl[]=$value;
}

# Check IP addresses sent
unset($nip, $nip_p);
for($i=0;$i<count($gip);$i++) {
	list($host,$port)=preg_split("/\:/",trim($gip[$i]),2);
	
	if( strcmp(long2ip(ip2long($host)),$host)==0 && is_numeric($port) && $port>0 ) {
		$nip[]=$host;
		$nip_p[]=$port;
	}
	else {
		$warnings.="WARNING: IP '$gip[$i]' invalid, dropped\n";
	}
}

# Check URLs sent
unset($nurl);
if( $enable_url!=0 ) { # URL sumbission is enabled
for($i=0;$i<count($gurl);$i++) {
	$isbad=0;
	$gurl[$i]=trim($gurl[$i]);
	if( !preg_match("/^http\:\/\//i",$gurl[$i]) ) { #TODO: better verification with parse_url
		$isbad=1;
	}
	if( $isbad==0 && $enable_http!=0 ) {
		$test = @fopen($gurl[$i]."?ping=1","r");
		if( $test ) {
			$out=fread($test,4);
			if( strncmp($out,"PONG",4) )
				$isbad=1;
		}
		else {
			$isbad=1;
		}
	}
	if( $isbad==0 ) {
		$nurl[]=$gurl[$i];
	}
	else {
		$warnings.="WARNING: URL '$gurl[$i]' invalid, dropped\n";
	}
}
}
else {
	$warnings.="WARNING: URL submission is disabled on this script (".count($gurl)." URLs were submitted)";
}

# Update hosts list
if( $maxhosts>30 ) $maxhosts=30; # anti-idiot code
if( $maxhosts<5 ) $maxhosts=5;
if( count($nip)>0 ) {
	$cnt=0;
	for($i=$data[1];$i<$data[2];$i++) {
		for($j=0;$j<count($nip);$j++) {
			if( $nip_p[$j]>0 && strncmp($data[$i],$nip[$j],strlen($nip[$j]))==0 ) {
				$data[$i]=$nip[$j].":".$nip_p[$j];
				$nip_p[$j]=0;
				$cnt++;
			}
		}
	}
	if( $cnt<count($nip) ) {
		$hosts=array_splice($data, $data[1], $data[2]-$data[1]); # remove
		for($j=0;$j<count($nip);$j++) {
			if( $nip_p[$j]>0 ) {
				$hosts[]=$nip[$j].":".$nip_p[$j];
			}
		}
		if( count($hosts)>$maxhosts )
			$hosts=array_splice($hosts,-$maxhosts); # take max
		array_splice($data, $data[1], 0, $hosts); # insert
		$chng=count($hosts)-($data[2]-$data[1]);
		$data[2]+=$chng;
		$data[3]+=$chng;
	}	
}

# Update URL list
if( $maxurls>30 ) $maxurls=30; # anti-idiot code
if( $maxurls<5 ) $maxurls=5;
if( count($nurl)>0 ) {
	$cnt=0;
	for($i=$data[2];$i<$data[3];$i++) {
		for($j=0;$j<count($nurl);$j++) {
			if( strcmp($data[$i],$nurl[$j])==0 ) {
				$nurl[$j]="";
				$cnt++;
			}
		}
	}
	if( $cnt<count($nurl) ) {
		$urls=array_splice($data, $data[2], $data[3]-$data[2]); # remove
		for($j=0;$j<count($nurl);$j++) {
			if( strlen($nurl[$j])>0 ) {
				$urls[]=$nurl[$j];
			}
		}
		if( count($urls)>$maxurls )
			$urls=array_splice($urls,-$maxurls); # take max
		array_splice($data, $data[2], 0, $urls); # insert
		$data[3]+=count($urls)-($data[3]-$data[2]);
	}	
}

# Final Output
if( count($gip)+count($gurl)>0 ) {
	exitwarn("OK");
}
else {
	$dont_count=1;
	print_page();
}

}

finish_up();

# ##### Functions #####

function finish_up() {
	global $file_data, $data, $lock;
	global $enable_stats, $dont_count;
	
	# ## Stats
	if( $dont_count==0 && $enable_stats!=0 ) {
		$data[4]++;
		if( time()>$data[6] ) {
			$data[7]=$data[5];
			$data[6]=time()+3600;
			$data[5]=0;
		}
		$data[5]++;
	}
	
	# ## Save File
	$out = fopen($file_data,"w");
	for($i=0;$i<count($data);$i++) {
		if( strlen($data[$i])>0 )
			fwrite($out,$data[$i]."\n");
	}
	fclose($out);
	flock($lock, LOCK_UN);
	fclose($lock);
	exit;
}


function exitwarn($msg) {
	global $warnings;
	echo $msg."\n";
	echo $warnings;
	finish_up();
	exit;
}

function print_page() {
global $enable_stats, $VERSION, $url_self, $warnings;
?>
<HTML>
<HEAD>
	<TITLE>GWebCache</TITLE>
</HEAD>
<BODY>
<H1>GWebCache</H1>

<P><B>
This is a GWebCache web-based Gnutella host cache system script.
</B></P>

<P>
For more information visit the <A HREF="http://www.zero-g.net/gwebcache/">GWebCache website</A>.
</P>

<? if( $enable_stats!=0 ) { ?>
<P>
Or check out some <A HREF="<?echo $url_self ?>?stats=1">stats</A> on this script.
</P>
<? } ?>

<P><SMALL>
<?echo $VERSION ?>
</SMALL></P>

<!-- Warnings:
<?echo $warnings ?>

-->

</BODY>
</HTML>
<?
}

function print_stats() {
global $enable_stats, $data, $maxhosts, $maxurls, $url_self;
?>
<HTML>
<HEAD>
	<TITLE>GWebCache Stats</TITLE>
</HEAD>
<BODY>
<H1>GWebCache Stats</H1>

<P>
<? if( $enable_stats!=0 ) { ?>
Total Requests: <?echo $data[4] ?><BR>
Requests This Hour: <?echo $data[5] ?> (since <?echo date("F j, Y, g:i a",$data[6]-3600) ?>)<BR>
Requests Last Hour: <?echo $data[7] ?> (average of <?echo $data[7]/60 ?> requests/min)<BR>
Hosts in Cache: <?echo $data[2]-$data[1] ?> (of <?echo $maxhosts ?>)<BR>
Alternate Caches: <?echo $data[3]-$data[2] ?> (of <?echo $maxurls ?>)<BR>
<? } else { ?>
Stats are diabled on this script.
<? } ?>
</P>

<P>
<A HREF="<?echo $url_self ?>">Back</A>
</P>

</BODY>
</HTML>
<?
}

?>
